/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/CustomerManager
 */
define(['baja!',
        'baja!electronicSignature:Customer,web:WebProfileConfig',
        'bajaux/mixin/subscriberMixIn',
        'Promise',
        'underscore',
        'nmodule/webEditors/rc/fe/baja/util/DepthSubscriber',
        'nmodule/webEditors/rc/fe/baja/util/typeUtils',
        'nmodule/webEditors/rc/fe/registry/StationRegistry',
        'nmodule/webEditors/rc/servlets/password',
        'nmodule/webEditors/rc/wb/mgr/Manager',
        'nmodule/webEditors/rc/wb/mgr/MgrTypeInfo',
        'nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand',
        'nmodule/webEditors/rc/wb/mgr/commands/NewCommand',
        'nmodule/webEditors/rc/wb/mgr/commands/RemoveCommand',
        'nmodule/webEditors/rc/wb/commands/DeleteCommand',
        'nmodule/webEditors/rc/wb/mgr/commands/MgrDuplicateCommand',
        'nmodule/webEditors/rc/wb/mgr/model/mgrAgents',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
        'nmodule/webEditors/rc/wb/mgr/model/columns/NameMgrColumn',
        'nmodule/webEditors/rc/wb/mgr/model/columns/PropertyMgrColumn',
        'nmodule/webEditors/rc/wb/table/model/Column',
        'nmodule/webEditors/rc/wb/table/model/source/ContainerComponentSource',
        'nmodule/webEditors/rc/wb/util/user/AuthenticatorMgrColumn',
        'nmodule/webEditors/rc/wb/util/user/ExpirationMgrColumn',
        'nmodule/electronicSignature/rc/wb/mgr/model/CustomerManagerMgrModel',
        'nmodule/webEditors/rc/wb/menu/CommandGroupContextMenu',
        'nmodule/webEditors/rc/wb/mixin/ContextMenuSupport'], function (
         baja,
         types,
         subscribable,
         Promise,
         _,
         DepthSubscriber,
         typeUtils,
         StationRegistry,
         password,
         Manager,
         MgrTypeInfo,
         EditCommand,
         NewCommand,
         RemoveCommand,
         DeleteCommand,
         DuplicateCommand,
         mgrAgents,
         MgrModel,
         NameMgrColumn,
         PropertyMgrColumn,
         Column,
         ContainerComponentSource,
         AuthenticatorMgrColumn,
         ExpirationMgrColumn,
         CustomerManagerMgrModel,
         CommandGroupContextMenu,
         addContextMenuSupport) {

  'use strict';

  var getSuperTypeChain = typeUtils.getSuperTypeChain,

      EDITABLE = Column.flags.EDITABLE,
      UNSEEN = Column.flags.UNSEEN,

      CUSTOMER_TYPE = baja.lt('electronicSignature:Customer'),

      TYPES_TO_PRELOAD = [
        'baja:AbsTime',
        'baja:Boolean',
        'baja:Date',
        'baja:DynamicEnum',
        'baja:Facets',
        'baja:Integer',
        'baja:Ord',
        'baja:Password',
        'baja:PasswordAuthenticator',
        'baja:String',
        'baja:Time',
        'baja:TypeSpec',
        'baja:UserPasswordConfiguration',
        'baja:Vector',
        'web:WebProfileConfig',
        'web:MobileWebProfileConfig',
        'webEditors:AuthenticationSchemeEditor',
        'webEditors:ExpirationEditor',
        'webEditors:RolesEditor',
        'webEditors:TimeFormatEditor',
        'webEditors:UserFacetsEditor',
        'webEditors:UserPasswordEditor',
        'webEditors:UserPrototypeEditor'
      ];

  //TODO: this should be more service-oriented in future.
  /**
   * As soon as the CustomerManager starts initializing, it will kick off a request
   * to import all the types its Edit dialog needs, then pre-populate the
   * StationRegistry with JS and agent registration info. This avoids multiple
   * hits to the registry servlets as the Edit property sheet loads, layer by
   * layer.
   *
   * @inner
   * @returns {Promise}
   */
  function preloadTypes() {
    return typeUtils.importTypes(TYPES_TO_PRELOAD)
      .then(function (types) {
        var superTypes = _.flatten(_.map(types, getSuperTypeChain)),
            typeSpecs = _.unique(_.map(superTypes, String)),
            reg = StationRegistry.getInstance();
        return Promise.all(_.map(typeSpecs, function (typeSpec) {
          return reg.queryAll(typeSpec);
        }));
      });
  }

  function getSelectedRows(manager) {
    return manager.getMainTable().getSelectedRows();
  }

  function isMultiSelection(manager) {
    return getSelectedRows(manager).length > 1;
  }

////////////////////////////////////////////////////////////////
// Column definitions
////////////////////////////////////////////////////////////////

  function toColumn(slot, params) {
    params = params || {};
    var Ctor = params.ctor || PropertyMgrColumn,
        flags = params.flags || EDITABLE;

    return new Ctor(slot, { type: CUSTOMER_TYPE, flags: flags });
  }

  var COLUMNS = [
    new NameMgrColumn({ flags: UNSEEN }),
    toColumn('customerName'),
    toColumn('address'),
    toColumn('customerPhoneNumber'),
    toColumn('contactPersonName'),
    toColumn('contactPersonPhoneNumber'),
  ];

////////////////////////////////////////////////////////////////
// CustomerManager
////////////////////////////////////////////////////////////////

  /**
   * User Manager.
   *
   * @private
   * @class
   * @alias module:nmodule/electronicSignature/rc/wb/mgr/CustomerManager
   * @extends module:nmodule/webEditors/rc/wb/mgr/model/MgrModel
   * @param {Object} [params]
   */
  var CustomerManager = function CustomerManager(params) {
    Manager.call(this, _.extend({
      moduleName: 'electronicSignature',
      keyName: 'CustomerManager'
    }, params));
    this.$subscriber = new DepthSubscriber(3);

    this.getCommandGroup().add(
      new EditCommand(this)
    );

    subscribable(this);
    addContextMenuSupport(this);

  };
  CustomerManager.prototype = Object.create(Manager.prototype);
  CustomerManager.prototype.constructor = CustomerManager;

  /**
   * Get the command to add new users.
   *
   * @private
   * @returns {module:bajaux/commands/Command}
   */
  CustomerManager.prototype.$getNewCommand = function () {
    return null;
  };

  /**
   * Get the command to edit the selected user(s).
   *
   * @private
   * @returns {module:bajaux/commands/Command}
   */
  CustomerManager.prototype.$getEditCommand = function () {
    return this.getCommandGroup().get(0);
  };

  /**
   * Get the command to remove the selected user(s).
   *
   * @private
   * @returns {module:bajaux/commands/Command}
   */
  CustomerManager.prototype.$getRemoveCommand = function () {
    return null;
  };

  /**
   * Get the command to duplicate the selected user(s).
   *
   * @private
   * @returns {module:bajaux/commands/Command}
   */
  CustomerManager.prototype.$getDuplicateCommand = function () {
    return null;
  };

  ////////////////////////////////////////////////////////////
  //Context Menu Support
  ////////////////////////////////////////////////////////////

  /**
   * Trigger context menus specifically on table row right clicks
   * @override
   * @returns {string}
   */
  CustomerManager.prototype.getContextMenuSelector = function () {
    return 'tr';
  };

  /**
   * Callback from ContextMenuSupport to provide a chance to pre-process
   * the right click menu's CommandGroup prior to displaying it
   *
   * @param group default CommandGroup consisting of subject's menu agents
   * @returns CommandGroup updated CommandGroup
   */
  CustomerManager.prototype.updateMenuCommandGroup = function (group) {
    if(group)
    {
      var deleteCmd = _.find(group.getChildren(), function (cmd)
      {
        return cmd instanceof DeleteCommand;
      });

      if (deleteCmd)
      {
        group.remove(deleteCmd);
      }

      var dupCmd = _.find(group.getChildren(), function (cmd)
      {
        return cmd instanceof MgrDuplicateCommand;  // jshint ignore:line
      });

      if (dupCmd)
      {
        group.remove(dupCmd);
      }

      var newCmd = _.find(group.getChildren(), function (cmd)
      {
        return cmd instanceof NewCommand;
      });

      if (newCmd)
      {
        group.remove(newCmd);
      }

      var multiSelect = isMultiSelection(this);

      if (multiSelect)
      {
        group.removeAll();
      }
    }

    return group;
  };

  /**
   * Returns array of currently selected subjects for context menu
   * @override
   * @returns {Array}
   */
  CustomerManager.prototype.getSubject = function () {
    return _.map(getSelectedRows(this), function (row) {
      return row.getSubject();
    });
  };

  /////////////////////////////////////////////////////////////////

  /**
   * When a row is double-clicked, invoke the Edit command for that row.
   * Enable the Duplicate option when one or more rows are selected
   *
   * @param {jQuery} dom
   * @returns {Promise}
   */
  CustomerManager.prototype.doInitialize = function (dom) {
    var that   = this,
        args   = arguments;

    preloadTypes(); //kick off types we know we need

    dom.on('dblclick', '.mainTable tr', function (e) {
      var subject = that.getSubject();
      console.log(subject); // jshint ignore:line
      //document.location.href = document.location.href.substring(0,document.location.href.indexOf("station:")) + subject[0].getNavOrd().toString().substring(subject[0].getNavOrd().toString().indexOf("station:"))+"%7Cview:electronicSignature:ZoneManager";
      var ord = baja.Ord.make(subject[0].getNavOrd().toString().substring(subject[0].getNavOrd().toString().indexOf("station:")).toString() + "|view:electronicSignature:ZoneManager");
               niagara.env.hyperlink(ord); // jshint ignore:line
      //that.$getEditCommand().invoke();
    });

    return Manager.prototype.doInitialize.apply(that, args);
  };


    /**
     * Create a `UserManagerMgrModel` instance with the appropriate columns.
     *
     * @param {baja.Component} userService
     * @returns {Promise.<module:nmodule/webEditors/rc/wb/util/user/UserManagerMgrModel>}
     */
    CustomerManager.prototype.makeModel = function (securedDashboardConfiguration) {
        //return CustomerManagerMgrModel.make(securedDashboardConfiguration);
        return Promise.all([
            MgrTypeInfo.make([ CUSTOMER_TYPE ]),
            mgrAgents.getMixInColumns(CUSTOMER_TYPE)
        ])
        .spread(function (newTypes, mixinCols) {
            return new CustomerManagerMgrModel({
                columns: COLUMNS.concat(mixinCols),
                componentSource: new ContainerComponentSource({
                    container: securedDashboardConfiguration,
                    filter: [ 'electronicSignature:Customer' ]
                }),
                newTypes: newTypes
            });
        });
    };


    /**
   * We do a switcheroo here; a `Manager` can't load a SecuredDashboardConfiguration directly,
   * so we override `load()` (an uncommon practice) to transmogrify that
   * SecuredDashboardConfiguration into a `MgrModel` instance before passing it to
   * `Manager#load`.
   *
   * @param {baja.Component} userService `electronicSignature:SecuredDashboardConfiguration`
   * @returns {Promise}
   */
/*  CustomerManager.prototype.load = function (securedDashboardConfiguration) {
    var that = this, newTypes;


    return MgrTypeInfo.make([ 'electronicSignature:Customer' ])
      .then(function (typeInfos) {
        newTypes = typeInfos;
        return mgrAgents.$getAgentsForMixinsOn(CUSTOMER_TYPE);
      })
      .then(function (agents) {

        var mixinCols = _.flatten(_.map(agents, function (agent) {
              return agent.getColumns();
            })),
            model = new CustomerManagerMgrModel({
              columns: COLUMNS.concat(mixinCols),
              componentSource: new ContainerComponentSource({
                container: securedDashboardConfiguration,
                filter: [ 'electronicSignature:Customer' ]
              }),
              newTypes: newTypes
            });

        return Manager.prototype.doLoad.call(that, model);
      });
  };*/

    /**
     * Load the model into the main table and configure the commands.
     *
     * @param {baja.Component} userService - the `baja:UserService`
     * @returns {Promise}
     */
    CustomerManager.prototype.doLoad = function (securedDashboardConfiguration) {
        var that = this;
        return Manager.prototype.doLoad.apply(that, arguments);
    };


  /**
   * Clean up the `MgrModel` itself as well as clean up the DOM.
   * @returns {Promise}
   */
  CustomerManager.prototype.doDestroy = function () {
    var model = this.value();
    //TODO: proper way of removing column event handlers
    /*if (model instanceof MgrModel) {
      model.removeColumns(model.getColumns());
    }*/
    return Manager.prototype.doDestroy.apply(this, arguments);
  };

  return CustomerManager;
});
