/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * A module that handles the management details for alarms in
 * a station Alarm Database
 * @module nmodule/alarm/rc/db/DatabaseMaintenance
 * @private
 */
define(['bajaux/Widget',
    'baja!',
    'baja!alarm:AlarmRecord,bql:DynamicTimeRange,bql:AbsTimeFilter,alarm:AlarmService',
    'jquery',
    'underscore',
    'Promise',
    'dialogs',
    'lex!alarm',
    'log!nmodule.alarm.rc.db.DatabaseMaintenance',
    'nmodule/webEditors/rc/fe/fe',
    'nmodule/webEditors/rc/fe/BaseWidget',
    'bajaux/events',
    'nmodule/alarm/rc/db/editors/Pager',
    'nmodule/electronicSignature/rc/wb/mgr/commands/RunCommand',
    'nmodule/alarm/rc/db/commands/NotesCommand',
    'nmodule/alarm/rc/db/commands/RefreshCommand',
    'nmodule/alarm/rc/db/commands/FilterCommand',
    'nmodule/alarm/rc/db/commands/AlarmDetailsCommand',
    'nmodule/alarm/rc/db/commands/PlayCommand',
    'hbs!nmodule/alarm/rc/db/templates/view',
    'css!nmodule/alarm/rc/db/templates/view'
], function (Widget, baja, types, $, _, Promise, dialogs, lexs, log, fe, BaseWidget,
             events,
             Pager,
             RunCommand, NotesCommand, RefreshCommand, FilterCommand,
             AlarmDetailsCommand, PlayCommand,
             viewTemplate) {

    'use strict';

    var lex                               = lexs[0],
        logSevere                         = log.severe.bind(log),
        CLEAR_OLD_ALARMS_LABEL            = lex.get('command.clearOldRecords.label'),
        CLEAR_SELECTED_ALARMS_LABEL       = lex.get('command.clearSelectedRecords.label'),
        CLEAR_BEFORE_SELECTED_ALARM_LABEL = lex.get('command.clearRecordsBeforeSelected.label'),
        CLEAR_ALL_ALARMS_LABEL            = lex.get('command.clearAllRecords.label'),
        RESULTS_PER_PAGE_LABEL            = lex.get('alarm.db.resultsPerPage'),
        DATE_EDITOR_CLASS                 = 'dateEditor',
        DATE_EDITOR_SELECTOR              = '.' + DATE_EDITOR_CLASS,
        SHELL_CONTAINER_SELECTOR          = '.bajaux-container',
        SHELL_TOOLBAR_SELECTOR            = '.bajaux-toolbar',
        TABLE_CONTAINER_SELECTOR          = '.tableContainer',
        TIME_RANGE_EDITOR_SELECTOR        = '.timeRangeEditor',
        MAINTENANCE_GROUP_SELECTOR        = '.maintenance',
        RUN_BTN_SELECTOR                  = '.run',
        PAGING_SELECTOR                   = '.paging',
        ALARM_TABLE_SELECTOR              = '.alarmTable',
        secureDashboardConfigObj          = null;

    function initCommandGroup(view) {
        var cmdGroup = view.getCommandGroup();
        _.each([
            PlayCommand.makeFor(view),
            FilterCommand.makeFor(view),
            AlarmDetailsCommand.makeFor(view),
            new NotesCommand(view),
            RefreshCommand.makeFor(view)
        ], function (cmd) {
            if (cmd instanceof NotesCommand && !view.isMaintenanceView()) {
                return;
            }
            cmdGroup.add(cmd);
        });

        return view;
    }

  function initTimeRangeEditor(view) {
    return baja.TimeZoneDatabase.get()
      .then(function (database) {
        var timezoneId = Intl.DateTimeFormat().resolvedOptions().timeZone,
          timezone = timezoneId && database.getTimeZone(timezoneId);
        return fe.buildFor({
          dom: $(TIME_RANGE_EDITOR_SELECTOR, view.jq()),
          value: baja.$('bql:DynamicTimeRange').make('today'),
          formFactor: 'mini',
          properties: {  TimeZone: timezone.encodeToString() }
        });
      })
      .then(function (editor) {
        view.timeRangeEditor = editor;
        editor.jq().on(events.MODIFY_EVENT, function (e, ed) {
          var pager = view.pager;
          if (pager) {
            ed.read()
              .then(function (timeRange) {
                pager.setTimeRange(timeRange);
                pager.setCurrentPage(1);
              })
              .catch(logSevere);
          }
          return false;
        });
        return view;
      });
  }

    function initDateEditor(view) {

        if (!view.isMaintenanceView()) {
            return view;
        }

        var today = new Date(),
            year  = today.getFullYear() - 5,
            month = today.getMonth(),
            day   = today.getDate();
        return fe.buildFor({
            dom: $(DATE_EDITOR_SELECTOR, view.jq()),
            value: baja.AbsTime.make({jsDate: new Date(year, month, day)}), //NCCB-14801
            formFactor: 'mini'
        }).then(function (editor) {
            editor.jq().on(events.MODIFY_EVENT, function () {
                return false;
            });
            view.dateEditor = editor;
            return view;
        });
    }

    function initRunMaintenanceButton(view) {
        if (!view.isMaintenanceView()) {
            return view;
        }
        return RunCommand.makeFor(view).then(function (button) {
            view.runBtn = button;
            return view;
        });
    }

    function initPaging(view) {
        return Pager.makeFor(view).then(function (pager) {

            //set bottom margin if db view
            if (!view.isMaintenanceView()) {
                $(PAGING_SELECTOR, view.jq()).css('margin-bottom', '20px');
            }

            view.pager = pager;
            return view;
        });
    }

    /**
     * Resize the alarm table to occupy as much space as possible within the constraints
     * of the shell container
     */
    function resizeTable() {
        var dom               = $(SHELL_CONTAINER_SELECTOR),
            viewHeight        = dom.outerHeight(true),
            toolbarHeight     = $(SHELL_TOOLBAR_SELECTOR, dom).outerHeight(true),
            timeRangeHeight   = $(TIME_RANGE_EDITOR_SELECTOR, dom).outerHeight(true),
            maintenanceHeight = $(MAINTENANCE_GROUP_SELECTOR, dom).outerHeight(true) || 0,
            pagerHeight       = $(PAGING_SELECTOR, dom).outerHeight(true),
            tableHeight       = viewHeight - timeRangeHeight - toolbarHeight - pagerHeight - maintenanceHeight;

        $(TABLE_CONTAINER_SELECTOR, dom).height(tableHeight);
    }

    function getWidget(view, selector) {
        return $(selector, view.jq()).data('widget');
    }

  /**
   * A view that handles the management details for alarms in
   * a station Alarm Database
   *
   * @class
   * @alias module:nmodule/alarm/rc/db/DatabaseMaintenance
   * @extends module:nmodule/webEditors/rc/fe/BaseWidget
   * @param params
   */
  var DatabaseMaintenance = function DatabaseMaintenance(params) {
    BaseWidget.call(this, _.extend({
      moduleName: 'alarm',
      keyName: 'DatabaseMaintenance'
    }, params || {}));
  };

    //extend and set up prototype chain
    DatabaseMaintenance.prototype = Object.create(BaseWidget.prototype);
    DatabaseMaintenance.prototype.constructor = DatabaseMaintenance;

    DatabaseMaintenance.prototype.$getTimeRangeEditor = function () {
        return getWidget(this, TIME_RANGE_EDITOR_SELECTOR);
    };

    DatabaseMaintenance.prototype.$getDateEditor = function () {
        return getWidget(this, DATE_EDITOR_SELECTOR);
    };

    DatabaseMaintenance.prototype.$getPager = function () {
        return this.pager;
    };

    DatabaseMaintenance.prototype.$getRunBtn = function () {
        return getWidget(this, RUN_BTN_SELECTOR);
    };

    /**
     * Return an array of the currently selected alarm records.
     *
     * @private
     *
     * @returns {Array} An array of alarm records.
     */
    DatabaseMaintenance.prototype.$getCurrentRecords = function() {
        var table = getWidget(this, ALARM_TABLE_SELECTOR),
        	rows = table.getSelectedRows();
    	return _.invoke(rows, 'getSubject');
    };

    DatabaseMaintenance.prototype.isMaintenanceView = function () {
        return true;
    };

    /**
     * Initial DOM setup
     *
     * @param {jQuery} dom the DOM element into which to load this View
     * @return {Promise} Promise
     */
    DatabaseMaintenance.prototype.doInitialize = function (dom) {

        var that   = this,
            widget = dom.data('widget');

        //NCCB-15117 - Destroy existing widget if present in DOM data
        return Promise.resolve(baja.hasType(widget, Widget) && !widget.isDestroyed() && widget.destroy())
            .then(function () {

                //load the view template
                dom.html(viewTemplate({
                    maintenanceView: that.isMaintenanceView(),
                    clearOldId: that.generateId(),
                    clearOld: CLEAR_OLD_ALARMS_LABEL,
                    clearSelectedId: that.generateId(),
                    clearSelected: CLEAR_SELECTED_ALARMS_LABEL,
                    clearBeforeSelectedId: that.generateId(),
                    clearBeforeSelected: CLEAR_BEFORE_SELECTED_ALARM_LABEL,
                    clearAllId: that.generateId(),
                    clearAll: CLEAR_ALL_ALARMS_LABEL,
                    resultsPerPage: RESULTS_PER_PAGE_LABEL
                }));

                //NCCB-14806: Disable text-highlighting in IE
                dom.on('selectstart', false);

                /*global window*/
                $(window).resize(resizeTable);

                return Promise.join(
                    initCommandGroup(that),
                    initTimeRangeEditor(that),
                    initDateEditor(that),
                    initRunMaintenanceButton(that),
                    initPaging //handler callback
                );
            });
    };

    DatabaseMaintenance.prototype.doLoad = function (secureDashboardConfigComp) {
        secureDashboardConfigObj = secureDashboardConfigComp;
        RunCommand.setObj(secureDashboardConfigObj);
        var that = this;
    	if (!baja.hasType(secureDashboardConfigComp, 'alarm:AlarmService')) {
       		 // get the alarmService and operate on that
      		 return baja.Ord.make('service:alarm:AlarmService').get().then(function(service) {
        	that.jq().find(MAINTENANCE_GROUP_SELECTOR).toggle(service.getPermissions().hasAdminInvoke());
        	resizeTable();
          });
        } else {
          that.jq().find(MAINTENANCE_GROUP_SELECTOR).toggle(secureDashboardConfigComp.getPermissions().hasAdminInvoke());
          resizeTable();
        }
    };

    /**
     * Destroy the relevant field editors
     * @param params
     */
    DatabaseMaintenance.prototype.doDestroy = function () {
        return this.getChildWidgets().destroyAll();
    };

    return DatabaseMaintenance;
});
