/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * A module responsible for running maintenance operations on the
 * histories selected by the 'manager' module
 *
 * API Status: **Private**
 * @module nmodule/history/rc/maintenance
 * @module nmodule/history/rc/ProtectedMaintenance
 * PROSVC-9806 Unable to delete history record from protected history database maintenance view.
 * If user has read only access to history space then user was not able to clear the history records
 * from protected database maintenance view on electronic signature ,so now we are invoking
 * station side action to do the same.
 * */
define([
  'baja!',
  'underscore',
  'Promise',
  'dialogs'
], function (baja, _, Promise, dialogs) {

  "use strict";
  var spacePromise,
      HISTORY_SCHEME = 'history:',
      BATCH_SIZE = 5;

  /**
   *  Resolve and returns the history space
   * @returns {Promise} Promise with resolved space
   */
  function getSpace() {
    return spacePromise || (spacePromise = baja.Ord.make(HISTORY_SCHEME).get());
  }

  function spaceUnresolved(err) {
    dialogs.showOk(err);
  }

  /**
   * Clears records from the history database prior to a given baja.AbsTime
   * @param ordList Array of type baja.Ord
   * @param beforeTime Time (baja.AbsTime) prior to which records should be cleared
   * @returns {Promise} Promise
   * PROSVC-9806 Unable to delete history record from protected history database maintenance view.
   * If user has read only access to history space then user was not able to clear the history records
   * from protected database maintenance view on electronic signature ,so now we are invoking
   * station side action to do the same.
   */
  function clearOldRecords(ordList, securedDashboardComp, beforeTime) {
    async function spaceResolved(space) {
      // return space.clearOldRecords(ordList, beforeTime);
      var comp=await securedDashboardComp.loadSlots();
      var historyMaintenance = securedDashboardComp.getHistoryMaintenance();
      if (historyMaintenance != null) {
        if (historyMaintenance.getPermissions().hasAdminWrite() && historyMaintenance.getPermissions().hasAdminRead()
            && historyMaintenance.getPermissions().hasAdminInvoke()) {
          var historyDetailsComp = baja.$("baja:Component");
          var compOrdList=await historyDetailsComp.add({
            slot: "ordList",
            value: baja.$("baja:OrdList").make(ordList)
          });

          var compBeforeTime=await historyDetailsComp.add({
            slot: "beforeTime",
            value: beforeTime
          });

          return historyMaintenance.invoke({
            slot: "clearOldRecords",
            value: historyDetailsComp
          });
        }
      }
    }

    return getSpace().then(spaceResolved, spaceUnresolved);
  }

  /**
   * Clears all records from the history database for the given list of ords
   * @param ordList Array of type baja.Ord
   * @returns {Promise} Promise
   * PROSVC-9806 Unable to delete history record from protected history database maintenance view.
   * If user has read only access to history space then user was not able to clear the history records
   * from protected database maintenance view on electronic signature ,so now we are invoking
   * station side action to do the same.
   */
  function clearAllRecords(ordList, securedDashboardComp) {
    function spaceResolved(space) {
      return securedDashboardComp.loadSlots().then(function () {
        var historyMaintenance = securedDashboardComp.getHistoryMaintenance();
        if (historyMaintenance != null) {
          if (historyMaintenance.getPermissions().hasAdminWrite() && historyMaintenance.getPermissions().hasAdminRead()
              && historyMaintenance.getPermissions().hasAdminInvoke()) {
            return historyMaintenance.invoke({
              slot: "clearAllRecords",
              value: baja.$("baja:OrdList").make(ordList)
            });
          }
        }
        //return space.clearAllRecords(ordList);
      });
    }
    return getSpace().then(spaceResolved, spaceUnresolved);
  }

  /**
   * Deletes histories from the history database for the given list of ords
   * @param ordList Array of type baja.Ord
   * @returns {Promise} Promise
   * PROSVC-9806 Unable to delete history record from protected history database maintenance view.
   * If user has read only access to history space then user was not able to clear the history records
   * from protected database maintenance view on electronic signature ,so now we are invoking
   * station side action to do the same.
   */
  function deleteHistories(ordList,securedDashboardComp) {
    function spaceResolved(space) {

      return securedDashboardComp.loadSlots().then(function () {
        var hasAllRequiredPermissions = false;
        var historyMaintenance = securedDashboardComp.getHistoryMaintenance();
        if (historyMaintenance != null) {
          if (historyMaintenance.getPermissions().hasAdminWrite() && historyMaintenance.getPermissions().hasAdminRead()
              && historyMaintenance.getPermissions().hasAdminInvoke()) {
            hasAllRequiredPermissions = true;
          }
        }
        //split the ord list into batches of size BATCH_SIZE
        var batches = [],
            i = 0;

        while (i < ordList.length) {
          if (i + BATCH_SIZE >= ordList.length) {
            batches.push(ordList.slice(i));
          } else {
            batches.push(ordList.slice(i, i + BATCH_SIZE));
          }
          i += BATCH_SIZE;
        }

        return Promise.all(_.map(batches, function (batch) {
          if (hasAllRequiredPermissions) {
            return historyMaintenance.invoke({
              slot: "deleteHistories",
              //value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
              value: baja.$("baja:OrdList").make(batch)
            });
          }
          // return space.deleteHistories(batch);

        }));
      });
    }


    return getSpace().then(spaceResolved, spaceUnresolved);
  }
  //Public API
  return {
    getSpace: getSpace,
    clearOld: clearOldRecords,
    clearAll: clearAllRecords,
    deleteAll: deleteHistories
  };
});
